// KNIHOVNA pro obsluhu digitlnhoteplotnho idla DALLAS (DS18B20)
// autor : Petr Navrtil
// verze 1.3 - C - soubor

#include "DS18B20.h"

// nastaven hodnoty spnn alarmu v teplotnm idlu
unsigned char T_H = 0x7F;				// horn teplota
unsigned char T_L = 0xF0;				// spodn teplota


// samotn podprogramy ----------------------------------------------------------------
int16_t Read_Teplota (void)
{
	int16_t hodnota;
	hodnota =0;
	reset();
	Write_Bus(0xcc);				// peskoen adresace idla
	Write_Bus(0x44);				// pkaz pro zahjen men teploty

	_delay_ms(200);					// Maximln prodleva pi rozlien 12 bit => 750 ms
	_delay_ms(200);					
	_delay_ms(200);
	_delay_ms(150);
	
	reset();
	Write_Bus(0xcc);				// pesko adresaci idla
	Write_Bus(0xbe);				// poadavek na vyten teploty (prvnch 16 bit)

  	for(unsigned char i = 0;i<16;i++)			// zachycen dat
			{
				hodnota = cteni(i,hodnota);				// vyti bit a ulo do promnn hodnota
				_delay_us(1);			// nutn prodleva mezi rmci ten
			}
	return(hodnota);
}

void reset (void)					// reset puls
{
	DDRQ |= 0x02;					// vysla TX
	PORTQ &= 0xfd;					// nuluje PORT A
	_delay_us(480);					// nuluje po dobu 500us >480us
	DDRQ &=0xfd;						// pepne do naslouchn (uvoln linku) -> RX pjima
	_delay_us(70);					// pok 60us po naskoen impulsu a vysln odpovdi
	_delay_us(410);					// pok do konce komunikace reset pulsu
}

void zapis_1 (void)					// zpis logick 1 na sbrnici
{
	DDRQ |= 0x02;					// dr na 0 po 6 us
	PORTQ &= 0xfd;
	_delay_us(6);
	DDRQ &= 0xfd;					// uvolni na zbytek do 70us, tedy na 64us
	_delay_us(64);
}
void zapis_0 (void)					// zpis logick 0 na sbrnici
{
	DDRQ |= 0x02;					// zapi logickou0 po 60us
	PORTQ &= 0xfd;
	_delay_us(60);
	DDRQ &= 0xfd;					// uvolni na 10us
	_delay_us(10);
}

int16_t cteni (unsigned char pozice, int16_t hodnota)				// zaznamen pchoz data do promnn hodnota
{
// pomocn promnn pro vpoty
	int16_t posun =1;
	DDRQ |= 0x02;					
	PORTQ &= 0xfd;					// vyli 0 na 6us
	_delay_us(6);
	DDRQ &= 0xfd;					// uvolni a pokej 9us
	_delay_us(9);
									// nati do pole aktuln hodnotu na sbrnici
	if ((PINQ & 0b00000010)!=0) hodnota|=(posun<<pozice);
		
	_delay_us(55);					// pokej do konce rmce

	return(hodnota);				// navrac zpt novou hodnotu teploty
}

void Write_Bus (int prikaz)				// zapisuje pkaz do teplotnho idla
{
	for(int i = 0; i<8;i++)			// prce pouze s 8 bitovmi povely (byte)
		{							// pole bit na i-t pozici, zan s LSB a kon s MSB
			if(((prikaz>>i)&0b00000001)==0) zapis_0();
				else zapis_1();
		}
}

void presnost_cidla (unsigned char nastav)				// zmna pesnosti men 0 a 3 (4 maximln rozlien 12 bit, 0 -> 9 bit)
{
		unsigned char presnost_mer;				    // pomocn promnn
		unsigned char presnost = 0b01111111;	

								// vyte z pesnost pouze 2 bity a pite 1
		presnost_mer=((presnost>>5)&0b00000011)+1;		

		if(presnost==4) presnost=0;	// zamezen pekroen rozsahu (00,01,10,11)

						// nastavm opt do stav registru a ten se nsledn pole do teplotnho idla
		presnost = (presnost_mer<<5)|0b00011111;		

		// aplikace zmny pesnosti
		zapis_scratch(presnost);
}

void zapis_scratch (unsigned char presnost)
{
		reset();						// reset sbrnice
		Write_Bus(0xcc);				// pesko adresaci
		Write_Bus(0x4e);				// zapi do tabulky "scratchpad" Th,Tl,conf. registr
		Write_Bus(T_H);					// hodnota Th
		Write_Bus(T_L);					// hodnota Tl
		Write_Bus(presnost);			// konfiguran registr - pesnost men(9,10,11,12 bit)
}

void alarm_zmena(unsigned char T_hor, unsigned char T_dol, unsigned char presnost)
{
	T_H = T_hor;    // zmn horn mez  
	T_L = T_dol;    // zmn doln mez
	zapis_scratch(presnost);  // zape do EEPROM => scratchpad
}

