// knihovna pro ENC28J60 - c. soubor
// autor: Petr Navrtil
// verze 1.0


#include "enc28j60.h"


//---------------------------------------------------------------------------------------------------------------
//------------------- Operace pro SPI sbrnici ------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------

inline void SPI_SendIns (unsigned char OPCODE, unsigned char argument)	// zape OPCODE - operaci, zape adresu
{
	char Data;

	CS_out;		// nastaven CS jako vstupn
	CS_clear;	// pod dobu operace drm CS na nulov rovni


	Data = (OPCODE<<5) + argument;		// OPCODE tvo prvn 3 bity, argument nsledujch 5

	/* Start transmission */
	SPDR = Data;				// zanu odeslat data

	/* Wait for transmission complete */
	while(!(SPSR & (1<<SPIF)));		// ekac smyka, kdy ekm na dokonen odesln

	// po spnm odesln musm vrtit CS na stav log 1 !!!!
}


// zale data na vstup (bez oeten dcch dat, pouze penos)
// ----------------------------------------------------------------
inline void SPI_sendDATA(unsigned char Data)
{
	/* zanu odeslat data */
	SPDR = Data;				 

	/* ekac smyka, kdy ekm na dokonen odesln */
	while(!(SPSR & (1<<SPIF)));		 

	// po spnm odesln musm vrtit CS na stav log 1 (a chci ukonit penos)
}

// ten dat (bez oeten dcch dat, pouze ten)
// -----------------------------------------------------------------------
inline unsigned char SPI_readDATA(void)
{
	/* poslm data pro generovn hodinovho signlu */
	SPDR = 0x00;	 

	/* ekac smyka, kdy ekm na dokonen odesln */
	while(!(SPSR & (1<<SPIF)));		

	/* Return data register */
	return SPDR;
}


void WriteENC (unsigned char OPCODE, unsigned char adresa, unsigned char *pozice, unsigned char delka )
{

	/* zpis do ENC OPCODE + adresa udv kam se bude zapisovat, pozice ukazuje na msto odkud se bude zuapisovat
	delka uruje kolik byt se zape za pkaz  */

	SPI_SendIns(OPCODE,adresa);		// provede nastaven vybran instrukce a adresy, CS je stle v log 0
		
	//v cyklu provedu zpis z pole po bytech do ENC28J60
	unsigned char pocitadlo =0;

	while(pocitadlo<delka)			// provd dokud nedoshne potebnho mnostv dat
	{
		SPI_sendDATA(*pozice);		// posl data na dan pozici
		pozice++;					// inkrementace ukazatele na data
		pocitadlo++;				// inkrementace potu vyslanch byt
	}

	// dokoncen operace penosu CS -> do log. 1
	CS_set;
	
} 


void ReadENC (unsigned char OPCODE, unsigned char adresa, unsigned char *pozice, unsigned char delka )
{
	/* ten ze sbrnice, prvn se odele pkaz pro ENC, nsleduje ten. Dlka uruje kolik byt se m uloit
	a pozice kma se maj uloit	*/

	SPI_SendIns(OPCODE,adresa);		// provede nastaven, CS nevypne

	// po vysln instrukce pepnu reim a zanu naslouchat na SI pinu	mikrokontrolru
	//v cyklu vytm do pole po bytech z ENC28J60 pouze poet byt zadanch v dlce
	unsigned char pocitadlo =0;

	while(pocitadlo<delka)	//provd dokud nemm potebn poet byt
	{
		*pozice = SPI_readDATA();	// pijm data 
		pozice++;
		pocitadlo++;
	}

	// dokoncen operace penosu CS -> do log. 1
	CS_set;
} 

//---------------------------------------------------------------------------------------------------------------
//------------------- Registrov operace ------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------

void writeReg (unsigned char OPCODE, unsigned char adresa, unsigned char data)
{
	// zasln instrukce
	SPI_SendIns(OPCODE,adresa);

	// zasln dat
	SPI_sendDATA(data);
	
	CS_set;	// ukonen operace
	
}

unsigned char readReg (unsigned char OPCODE, unsigned char adresa)
{
	SPI_SendIns (OPCODE,adresa);
	SPI_readDATA();
	CS_set;					// ukonen operace
	return SPDR;			// vrac hodnotu registru
	
}

void SetBank (unsigned char banka)			// nastaven sprvn registov banky
{
	stavReg = readReg(ReadConReg,ECON1);	// natu hodnotu registru
	stavReg = (stavReg&0xFC)|banka;			// nastavm BEL1 a BEL0 na banku 0,1,2,3 - dle vbru
	writeReg (WriteConReg,ECON1,stavReg); 	// zapi nov hodnoty do ECON1
}

void TxReset (void)							// reset odeslac logiky
{
//	unsigned char stavReg;
	stavReg = readReg(ReadConReg,ECON1);
	stavReg = stavReg|0x80;					// nastavm MSB pro reset logiky
	writeReg (WriteConReg,ECON1,stavReg);

	// nulovn TXRST
	stavReg = readReg(ReadConReg,ECON1);
	stavReg = stavReg & 0x7F;				// nuluje MSB
	writeReg (WriteConReg,ECON1,stavReg);
}

void RxReset (void)							// reset odeslac logiky
{
//	unsigned char stavReg;
	stavReg = readReg(ReadConReg,ECON1);
	stavReg = stavReg|0x40;					// nastavm MSB-1 pro reset logiky
	writeReg (WriteConReg,ECON1,stavReg);

	// nulovn RXRST
	stavReg = readReg(ReadConReg,ECON1);
	stavReg = stavReg & 0xBF;				// nuluje MSB-1
	writeReg (WriteConReg,ECON1,stavReg);
}


void SysReset (void)						// systmov reset
{
	SPI_SendIns(7,0x1F);					// pole byte jedniek
	CS_set;									// ukon tuto instrukci

}


void Init_ENC (unsigned char *poziceMAC)		// funkce kter inicializuje ENC -nastav vechny jeho potebn funkce
{
	

// po systmovm resetu nsleduje nastaven rychlosti SPI sbrnice (je dna procesorem -> vypnout CLK_out)
	SetBank(3);	// ECOCON se nachz v bance slo 3
	stavReg = 0b00000000; // posledn 3 bity (COCON2 1,0) stav 000 vypne CLKOUT
	writeReg (WriteConReg,ECOCON,stavReg);


//---------------------------------------------------------------------------------------------------------------
//------------------- PIJMAC ST - BUFFERU RX -----------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------


	SetBank(1);
// nastavm pijmac filtr
	// UNICAST (7), ANDOR (6), CRCEN (5), Patern Match FILTR (4), Magic Packet (3), Hash Table (2), Multicast (1), Broadcast (0)
	// nastaven UNICAST or BROADCAST se sprvnm CRC => UCEN =1,ANDOR = 0, CRCEN = 1, BCEN = 1; 
	stavReg = 0b00100001;
	writeReg (WriteConReg,ERXFCON,stavReg);


// nastaven pracovn pamti pro zpis a ten - nastaven jednotlivch ukazatel
	SetBank(0);
	// nastaven pijmacho bufferu (spodn bit, horn bit) - startovac adresa <erxstl>	- zane na 0x1A19
	stavReg = 0x19;
	writeReg (WriteConReg,ERXSTL,stavReg);	
	stavReg = 0x1A;
	writeReg (WriteConReg,ERXSTH,stavReg);

	// nastaven pijmacho bufferu (spodn bit, horn bit) - adresa ukazatele  <erxrdptl>
	stavReg = 0x19;
	writeReg (WriteConReg,ERXRDPTL,stavReg);	
	stavReg = 0x1A;
	writeReg (WriteConReg,ERXRDPTH,stavReg);

	// nastaven pijmacho bufferu (spodn bit, horn bit) - koncov adresa bufferu <erxndl>
	// samotn konec pamti
	stavReg = 0xFF;
	writeReg (WriteConReg,ERXNDL,stavReg);	
	stavReg = 0x1F;
	writeReg (WriteConReg,ERXNDH,stavReg);

//---------------------------------------------------------------------------------------------------------------
//------------------- ODESLAC ST BUFFERU TX -----------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------

	// nastaven odeslacho bufferu (spodn bit, horn bit) - startovac adresa <etxstl>
	stavReg = 0x00;
	writeReg (WriteConReg,ETXSTL,stavReg);
	stavReg = 0x00;
	writeReg (WriteConReg,ETXSTH,stavReg);

/* deklarace pouze zatku, zbytek a po zatek 1A19 je pam pro odeslac buffer, pouze uku ukazatelem na
msto a urm konec paketu pi vyslan */

//---------------------------------------------------------------------------------------------------------------
//------------------- NASTAVEN MAC VRSTVY -- (banka 2) ---------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------

// nastaven registru MACON1 
	SetBank(2);
	stavReg = 0b00001101;	// nastavm TXPAUS, RXPAUS, MARXEN (povol doruen na MAC)
	writeReg (WriteConReg,MACON1,stavReg);

// nastaven registru MACON3 (CRC operace)
	// pidvn nul ke krtkm rmcm PADCFG2-0 (001)< 60 byt doplnno 0>, povolen vpotu CRC TXCRCEN
	// kontrola FRMLNEN - zapnut
	stavReg = 0b00110010;
	writeReg (WriteConReg,MACON3,stavReg);

// nastaven registr MAIPGL, MAIPGH, MABBIPG (banka 2)
	stavReg = 0x12;	
	writeReg (WriteConReg,MABBIPG,stavReg);
	stavReg = 0x12;
	writeReg (WriteConReg,MAIPGL,stavReg);
	stavReg = 0x0C;
	writeReg (WriteConReg,MAIPGH,stavReg);

// maximln pijman a vyslan paket
	writeReg (WriteConReg,MAMXFLL,0x00);	// 256 byt max rmec 0x0040 (monost nastavit a 1500 Byt)
	writeReg (WriteConReg,MAMXFLH,0x01);

// nastaven MAC adresy
	SetBank(3);									// vbr banky slo 3

	writeReg(WriteConReg,MAADR6,*poziceMAC);	// zpis 1. bytu MAC
	poziceMAC--;
	writeReg(WriteConReg,MAADR5,*poziceMAC);	// zpis 2. bytu MAC
	poziceMAC--;
	writeReg(WriteConReg,MAADR4,*poziceMAC);	// zpis 3. bytu MAC
	poziceMAC--;
	writeReg(WriteConReg,MAADR3,*poziceMAC);	// zpis 4. bytu MAC
	poziceMAC--;
	writeReg(WriteConReg,MAADR2,*poziceMAC);	// zpis 5. bytu MAC
	poziceMAC--;
	writeReg(WriteConReg,MAADR1,*poziceMAC);	// zpis 6. bytu MAC


// nastaven registru ECON1
	// Povolen pjmu (RXEN, CRC povolen - CSUM, nastav banku 00)
	stavReg = 0b00010100; // TXRST, RXRST,DMAST, CSUMEN,TXRTS,RXEN, BSEL1, BSEL0
	writeReg (WriteConReg,ECON1,stavReg); // zapi nov hodnoty do ECON1

// nastaven registru ECON2
	// Automatick inkrementace ukazatele na buffer ERDPT nebo EWRPT (AUTOINC), PKTDEC dekrementuje EPKTCNT (log1)
	stavReg = 0b10000000; // AUTOINC, PKTDEC,PWRSV, r,VRPS
	writeReg (WriteConReg,ECON2,stavReg); // zapi nov hodnoty do ECON1

/*
	ENC m nyn nastaveno:
		- pozice v bufferu vyhrazeny pro ten i zpis
		- nastaveno pidn CRC vpotu a nastaven MAC vrstvy (dlka pakteu apod.)
		- Konfigurovan spojen Half-Duplex a SPI CLK_OUT je vypnut
		- definuje MAC adresu vloenou do funkce inicializace
*/

}

void WriteBuffer(unsigned char *pozicePAK,unsigned char pocet)			// zpis do buffru ENC maximln 57 znak
{
	// pozicePAK .... ukazatel na hodnotu od kter se maj zapsat data
	// pocet ........ pocet byt k vysln maximum 57B, zbytek je rezervovn do 60 byt pro ENC + CRC

	unsigned char pocitadlo =0;

	// nastaven ukazatele pro zpis
	SetBank(0);

	stavReg = 0x00;	// 0
	writeReg (WriteConReg,EWRPTL,stavReg);

	stavReg = 0x00;	// 0
	writeReg (WriteConReg,EWRPTH,stavReg);


	//zapsn OPCODE pro WRBuff, adresace bufferu a nsleduj data
	SPI_SendIns(WriteBuffMem,BuffAdress);		// provede nastaven, CS nevypne
		
	//v cyklu provedu zpis z pole po bytech do ENC28J60
	SPI_sendDATA(0x00);	// prvn byte v pamti nulov, nsledn se ukld paket z hlaviky IP

	while(pocitadlo < pocet)			// vype ve a do velikosti pocet
	{
		SPI_sendDATA(*pozicePAK);	// posl data na dan pozici
		pozicePAK++;				// inkrementuje ukazatel
		pocitadlo++;
	}


	//doplnn paketu nulami na velikost 60 byt
	pocitadlo =0;

	while(pocitadlo < (57-pocet))	
	{
		SPI_sendDATA(0x00);			//posl nulov data
		pocitadlo++;
	}

	// dokoncen operace penosu CS -> do log. 1
	CS_set;


}

void ReadBuffer(unsigned char *pozicePAK, uint16_t adresa)
{
	// pozicePAK .... ukazatel na hodnotu kam se zap data
	// pozice ....... pozice ukazatele

	SetBank(0);

	unsigned char pocitadlo =0;

	// reset pijmac logiky (nastaven a nulovn RXRST v ECON1)
	RxReset();

	// nastaven pijmacho bufferu (spodn bit, horn bit) - adresa ukazatele  <erxrdptl>
	stavReg = (unsigned char)(adresa);
	writeReg (WriteConReg,ERDPTL,stavReg);	
	stavReg = (unsigned char)(adresa>>8);
	writeReg (WriteConReg,ERDPTH,stavReg);

	//zapsn OPCODE pro ReadBuff, adresace bufferu a nsleduj data
	SPI_SendIns(ReadBuffMem,BuffAdress);		// provede nastaven CS a nevypne ho

	while( (pocitadlo < 60) )	
	{
		*pozicePAK = SPI_readDATA();	// vytu a zapi na danou pozici
		pozicePAK++;					// inkrementuje ukazatel
		pocitadlo++;
	}

	// dokoncen operace penosu CS -> do log. 1
	CS_set;
}

void ReadPacketData(unsigned char *pozicePAK, uint16_t adresa)
{
	SetBank(0);
	unsigned char pocitadlo =0;

	// reset pijmac logiky (nastaven a nulovn RXRST v ECON1)
	// RxReset();

	// nastaven pijmacho bufferu (spodn bit, horn bit) - adresa ukazatele  <erxrdptl>
	stavReg = (unsigned char)(adresa&0x00FF);
	writeReg (WriteConReg,ERDPTL,stavReg);	
	stavReg = (unsigned char)((adresa>>8)&0x00FF);
	writeReg (WriteConReg,ERDPTH,stavReg);

	//zapsn OPCODE pro ReadBuff, adresace bufferu a nsleduj data
	SPI_SendIns(ReadBuffMem,BuffAdress);		// provede nastaven CS a nevypne ho

	while( (pocitadlo < 6) )	
	{
		*pozicePAK = SPI_readDATA();	// vytu a zapi na danou pozici
		pozicePAK++;					// inkrementuje ukazatel
		pocitadlo++;
	}

	// dokoncen operace penosu CS -> do log. 1
	CS_set;

}


void SendPacket (unsigned char pozice)			// odele paket uloen na dan adrese v bufferu
{
//	unsigned char stavReg;

	// reset vyslac logiky (nastaven a nulovn TXRST)
	TxReset();

	// nastavm ukazatel na TX paketu na zatek prostoru ewrptl, ewrpth
	writeReg (WriteConReg,EWRPTL,pozice);
	writeReg (WriteConReg,EWRPTH,0x00);

	// nastavm konec paketu
	stavReg = 0x2E;			// (60B)
	writeReg (WriteConReg,ETXNDL,stavReg);
	stavReg = 0x00;
	writeReg (WriteConReg,ETXNDH,stavReg);

	// odeslm nastavenm TXRTS v ECON1
	stavReg = readReg(ReadConReg,ECON1);
	stavReg |=0b00011000;		// nastavm MSB TXRTS na hodnotu 1 a odelu paket, nastaven CSUM
	writeReg (WriteConReg,ECON1,stavReg); // zapi nov hodnoty do ECON1

}

