#include <avr/io.h>
#include "enc28j60.h"
#include <util/delay.h>
#include "DS18B20.h"

void CalculateCRC (void);				// kalkulace CRC
void vysli_UART (unsigned char *vypis, unsigned char delka);	// pomocn fce. pro UART
void ConfigureIP (void);				// zmna IP a MAC

unsigned char *poslat_Data;
unsigned char *prijato_Data;
unsigned char *register_Data;
unsigned char *status_Data;

uint16_t teplo = 0;
uint16_t adresace = 0x1A19;


unsigned char Prijato[60];				// pole pro pjem paketu
unsigned char StatusVektor[6]={0};		// 4.pole vektoru udv velikost paketu ==> ukazatel +(dlka+1) = adresa dalho paketu
unsigned char Packet[45] = 	"\xFF\xFF\xFF\xFF\xFF\xFF\2\4\5\1\2\6" // MAC address (pjemce, odeslatel)
	       								"\x08\x00" // Ethernet freame type
//zatek IP rmce-------------------------------------------------------
//-----------------------------------------------------------------------
//1. dek IP hlaviky: verze+IHL; Typ sluby; celkov dlka (2bajty)		
			 "\x45\x00\x00\x2D"	
//2. dek IP hlaviky: Idetifikace (2B); Offset fragmentu(2B)
			"\x00\x01\x00\x00"
//3. dek IP hlaviky: TTL;Protokol; CRC hlaviky (2B)  <= doplnit CRC kd
			"\x80\x11\x00\x00"		
//4. dek IP hlaviky: Zdrojov adresa
			"\x0A\x00\x00\x09"		// 10.0.0.9
//5. dek IP hlaviky: clov adresa
			"\x0A\x00\x00\x65"		// 10.0.0.1  (101)
//UDP hlavika-----------------------------------------------------------
//-----------------------------------------------------------------------
//1. dek UDP: zdrojov port(2B), clov port(2B)  //port 55 555		
			"\xD9\x03\xD9\x03"
//2. dek UDP: dlka(2B), CRC(2B)
			"\x00\x19\x00\x00"
//3. dek UDP: data (2B)
			"\x08\xAA";		// vyle DATA = slo 88 - signalizace, e nejde teplomr;


// inicializace SPI rozhran pro komunikaci
void init_SPI(void)
{	
	DDRB = (1<<PB4)|(1<<PB5) | (1<<PB7);	 // nastav MOSI , SCK , and SS jako vstup
	SPCR = ( (1<<SPE)|(1<<MSTR)  );		// Enable SPI, Master, set clock rate fck = 8MHz
}


void init_uart (void)		// debuging pes UART 9600,1-stop bit, N/A parita
{
	UCSRA = 0b00100000;
	UCSRB = 0b10011000;
	UCSRC = 0b10000110;
	UBRRL = 103;
}


//------ hlavn program -----------------------------------------------------------------
int main (void)
{
	CS_out;					// DDRB pin SS je uren jako CS (viz. knihovna enc28J60.h)
	CS_set;					// CS nastaven do log 1
	DDRC = 0xFF;			// nastaven portu C jako vstupnho pro periferie na desce
	PORTC = 0b01000011;		// nastaven periferi na desce

	init_SPI();				// incializace SPI rozhran
	init_uart();			// nastavm UART pro DEBUG

	// ukazatel na data
	poslat_Data = &Packet[0];
	prijato_Data = &Prijato[0];
	status_Data = &StatusVektor[0];
	register_Data = &Packet[11]; // v prvn ad nastav 6. byte (pozice MAC pro ENC)


	// inicializace (restart) ENC
	SysReset();
	_delay_ms(10);

	// MAC adresa se ulo do ENC
	Init_ENC(register_Data);

	// vpoet CRC pro vchoz IP hlaviku
	CalculateCRC();	


	while(1)
	{
		/* vyten teploty z teplotnho idla (standartn 12 bit) */
		teplo=Read_Teplota();	
				
		/* pozice dat pro teplotu 42 a 43 byte protokolu*/
		Packet[42] = (unsigned char)(teplo>>8);
		Packet[43] = (unsigned char)(teplo);
		
		WriteBuffer(poslat_Data,44);	// zape data k odesln do bufferu v ENC, MAX paket je nastaven na 60 byt
		SendPacket(0x00);				// zape paket na dan adrese 0x00xx,. kde xx je pozice zatku paketu (nejjednodu ppad na zatku 0x00)

		// wait cca 2 sekundy ------------------
		_delay_ms(200);		// 1 sekunda
		_delay_ms(200);
		_delay_ms(200);
		_delay_ms(200);
		_delay_ms(200); 
		_delay_ms(200);		// 2 sekunda
		_delay_ms(200);
		_delay_ms(200);
		_delay_ms(200);
		_delay_ms(200); 
		/* --------------------------------- */

		ConfigureIP();					// otestuje pijat pakety na ppadnou zmnu IP adresy pjemce
	
	}

	return(1);

}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------
void CalculateCRC (void)
{
	//vymazat prvn kalkulaci!!!!  pozice 24,25
	Packet[24]=0x00;
	Packet[25]=0x00;

	uint16_t cisla[10];
	uint32_t soucet = 0;
	uint16_t CRC_kod;

	unsigned char i,preteceni;

	//slouen do 16 bitovch slov
	for(i=0; i<10;i++)
	{
		cisla[i]=(uint16_t)Packet[14+(2*i)];
		cisla[i]= cisla[i]<<8;
		cisla[i]+= Packet[15+(2*i)];
		//index 14 (1 IP) a 33 (20IP)
	}
	
	// provedeme souet vech 16 bitovch sel
	for(i=0; i<10;i++)
	{
		soucet+= cisla[i];
		
	}

	CRC_kod = (uint16_t)(0xFFFF&soucet); 		// uloen vsledek CRC
	preteceni = (unsigned char)(soucet>>16);	// vsledek peteen
	CRC_kod +=preteceni;						// piten peteen k hodnot CRC
	CRC_kod = ~CRC_kod;							// negace vslednho CRC kdu

	Packet[24] = (unsigned char)(0xFF&(CRC_kod>>8));
	Packet[25] = (unsigned char)(0xFF&(CRC_kod));

}

void ConfigureIP (void)
{
		/* Procedura na ten paketu a nastaven IP komunikace */
		// 1. zjitn novho paketu - peten status vektoru
		ReadPacketData(status_Data,adresace);			// petu status vektor pchozho paketu					// status vektor je vdy 4 znaky
		// kontrola vektoru paketu
		if ((StatusVektor[0]==0x60)&(StatusVektor[1]==0x1A))
		{
			//2. pokud nalezl paket peskom status vektor a petu buffer, oekvm 60 byt packet
			adresace = adresace+6;
			ReadBuffer(prijato_Data,(adresace));			// petu paket
//			vysli_UART (status_Data,6);						// status vektor je vdy 4 znaky
//			vysli_UART (prijato_Data,60); 
		
	
		// 3. Hledn danho vrazu v paketu od bytu 43 a po 47 (po 44)
			if ((Prijato[42]==0x53)&(Prijato[43]==0x54)&(Prijato[44]==0x41)&(Prijato[45]==0x52)&(Prijato[46]==0x54))	
			{
				// pokud je nalezeno slovo START => ascii {53,54,41,52,54}
				// jedn se o nastavovac paket a potebuji nast IP adresy a MAC adresu danho PC

				// zazen dostane na dan sti automaticky adresu s koncovkou 09
				// zmna IP pro danou s
				Packet[26]=Prijato[26];
				Packet[27]=Prijato[27];
				Packet[28]=Prijato[28];
				Packet[29]=0x09;

				// adresa kam poslat
				Packet[30]=Prijato[26];
				Packet[31]=Prijato[27];
				Packet[32]=Prijato[28];
				Packet[33]=Prijato[29];

				// zmna MAC adresy na kterou poslat -> ji nebude broadcast
				Packet[0]=Prijato[6];
				Packet[1]=Prijato[7];
				Packet[2]=Prijato[8];
				Packet[3]=Prijato[9];
				Packet[4]=Prijato[10];
				Packet[5]=Prijato[11];

				// probhla zmna rmce, je nutn pepotat CRC
				CalculateCRC();
				

			}
			else 		// pokud se nejedn o nastavovac paket, potom se v pamti posunu na dal paket
			{
				adresace = adresace+StatusVektor[3]-5;			// na zatek dalho paketu 
			}


		}
		else		// pokud nebyl nalezen zatek dalho paketu probh hledn znovu
		{
			adresace = 0x1A19;
		}
}


void vysli_UART (unsigned char *vypis, unsigned char delka)
{
	unsigned char i=0;

	for(i=0; i<delka;i++)
	{
	UDR = *vypis;
	vypis++;
	_delay_ms(3);
	}

}



